@@ -149,11 +149,11 @@ vips_Lab2LabQ_init( VipsLab2LabQ *Lab2LabQ )
 	colour->coding = VIPS_CODING_LABQ;
 	colour->interpretation = VIPS_INTERPRETATION_LABQ;
 	colour->format = VIPS_FORMAT_UCHAR;
+	colour->input_bands = 3;
 	colour->bands = 4;
 
 	code->input_coding = VIPS_CODING_NONE;
 	code->input_format = VIPS_FORMAT_FLOAT;
-	code->input_bands = 3;
 }
 
 /**
