@@ -208,10 +208,10 @@ vips_scRGB2sRGB_init( VipsscRGB2sRGB *scRGB2sRGB )
 	colour->coding = VIPS_CODING_NONE;
 	colour->interpretation = VIPS_INTERPRETATION_sRGB;
 	colour->format = VIPS_FORMAT_UCHAR;
+	colour->input_bands = 3;
 	colour->bands = 3;
 
 	code->input_coding = VIPS_CODING_NONE;
-	code->input_bands = 3;
 	code->input_format = VIPS_FORMAT_FLOAT;
 
 	scRGB2sRGB->depth = 8;
