@@ -166,9 +166,6 @@ vips_thread_profile_free( VipsThreadProfile *profile )
 {
 	VIPS_DEBUG_MSG( "vips_thread_profile_free: %s\n", profile->name ); 
 
-	if( vips__thread_profile ) 
-		vips_thread_profile_save( profile ); 
-
 	VIPS_FREEF( g_hash_table_destroy, profile->gates );
 	VIPS_FREEF( vips_thread_gate_free, profile->memory );
 	VIPS_FREE( profile );
@@ -183,20 +180,15 @@ vips__thread_profile_stop( void )
 
 static void
 vips__thread_profile_init_cb( VipsThreadProfile *profile )
-{  
-	/* Threads (including the main thread) must call 
-	 * vips_thread_shutdown() before exiting. Check that they have.
+{
+	/* We only come here if vips_thread_shutdown() was not called for this
+	 * thread. Do our best to clean up.
 	 *
-	 * We can't save automatically, because the shutdown order is
-	 * important. We must free all memory before saving the thread
-	 * profile, for example.
+	 * GPrivate has stopped working, be careful not to touch that. 
 	 *
-	 * We can't do the freeing in this callback since GPrivate has already
-	 * stopped working. 
+	 * Don't try to save: this is an emergency recovery path. 
 	 */
-	vips_warn( "VipsBuffer", 
-		_( "vips_thread_shutdown() not called for thread %p" ), 
-		g_thread_self() ); 
+	vips_thread_profile_free( profile );
 }
 
 static void
@@ -269,6 +261,9 @@ vips__thread_profile_detach( void )
 	VIPS_DEBUG_MSG( "vips__thread_profile_detach:\n" ); 
 
 	if( (profile = vips_thread_profile_get()) ) {
+		if( vips__thread_profile ) 
+			vips_thread_profile_save( profile ); 
+
 		vips_thread_profile_free( profile );
 		g_private_set( vips_thread_profile_key, NULL );
 	}
